## this file reads in various data files and recodes them so that the
## values of the variables are more easily understood.
##
## NB: this is tied tightly to the coding rules used in DataPrep.R
## any changes the coding of variables in DataPrep.R must be reflected
## with accompanying changes to OutputRecode()
##
## assumes x contains MiniColor and MiniLoc vars
##

source("utils.R")


load("XimpMat.Rdata") ## gets x.imp.mat
ExitPollRaw <- OutputRecode(x.imp.mat)
save(ExitPollRaw, file="ExitPollRaw.Rdata")

load("XBackground.Rdata")
x.background$BachDegPrecinct <- x.imp.mat[,"BachDegPrecinct"]
x.background$HHincomePrecinct <- x.imp.mat[,"HHincomePrecinct"]



impClean <- function(x, x.background, newname, file, keep.color){

  filestub <- strsplit(file, "\\.")[[1]][1]
  csvfile <- paste(filestub, ".csv\"", sep="")
  
  Imp <- OutputRecode(x)
  Imp <- cbind(Imp, x.background)
  Imp <- Imp[Imp$MiniColor %in% keep.color, ]
  keep.cols <- colnames(Imp) %in% c("MiniLoc.1", "MiniColor.1")
  drop.names <- c("TotalPop00.x",
                  "TotalVAP00",
                  "WhiteVAP00",
                  "BlackVAP00",
                  "HispanicVAP00",
                  "NatAmVAP00",
                  "AsianVAP00",
                  "RemainderVAP00")
  Imp <- Imp[, !keep.cols]
  drop.cols <- NULL
  ncol.imp <- ncol(Imp)
  for (i in 1:(ncol.imp-1)){
    if (colnames(Imp)[i] %in% drop.names){
      drop.cols <- c(drop.cols, i)
    }
    else{
      for (j in (i+1):ncol.imp){
        if (identical(Imp[,i], Imp[,j])){
          drop.cols <- c(drop.cols, j)
        }
      }
    }
  }
  Imp <- Imp[,-drop.cols]
  eval(parse(text=paste(newname, " <- Imp", sep="")))
  eval(parse(text=paste("save(", newname, ", file=", file, ")", sep="")))
  eval(parse(text=paste("write.csv(", newname, ", file=",
               csvfile, ")", sep="")))
  return(NULL)
}









 
load("ChoiceDemogImp1.Rdata")
impClean(filled.choice.demog.dat.1, x.background,
         "ChoiceImp1", "\"ChoiceImp1.Rdata\"", c("G", "Y"))
load("PinkDemogImp2-1.Rdata")
impClean(filled.pink.demog.dat.1, x.background,
         "PinkImp1", "\"PinkImp1.Rdata\"", c("P"))
load("BlueDemogImp2-1.Rdata")
impClean(filled.blue.demog.dat.1, x.background,
         "BlueImp1", "\"BlueImp1.Rdata\"", c("B"))







 
load("ChoiceDemogImp2.Rdata")
impClean(filled.choice.demog.dat.2, x.background,
         "ChoiceImp2", "\"ChoiceImp2.Rdata\"", c("G", "Y"))
load("PinkDemogImp2-2.Rdata")
impClean(filled.pink.demog.dat.2, x.background,
         "PinkImp2", "\"PinkImp2.Rdata\"", c("P"))
load("BlueDemogImp2-2.Rdata")
impClean(filled.blue.demog.dat.2, x.background,
         "BlueImp2", "\"BlueImp2.Rdata\"", c("B"))







 
load("ChoiceDemogImp3.Rdata")
impClean(filled.choice.demog.dat.3, x.background,
        "ChoiceImp3", "\"ChoiceImp3.Rdata\"", c("G", "Y"))
load("PinkDemogImp2-3.Rdata")
impClean(filled.pink.demog.dat.3, x.background,
         "PinkImp3", "\"PinkImp3.Rdata\"", c("P"))
load("BlueDemogImp2-3.Rdata")
impClean(filled.blue.demog.dat.3, x.background,
         "BlueImp3", "\"BlueImp3.Rdata\"", c("B"))







 
load("ChoiceDemogImp4.Rdata")
impClean(filled.choice.demog.dat.4, x.background,
         "ChoiceImp4", "\"ChoiceImp4.Rdata\"", c("G", "Y"))
load("PinkDemogImp2-4.Rdata")
impClean(filled.pink.demog.dat.4, x.background,
         "PinkImp4", "\"PinkImp4.Rdata\"", c("P"))
load("BlueDemogImp2-4.Rdata")
impClean(filled.blue.demog.dat.4, x.background,
         "BlueImp4", "\"BlueImp4.Rdata\"", c("B"))







 
load("ChoiceDemogImp5.Rdata")
impClean(filled.choice.demog.dat.5, x.background,
         "ChoiceImp5", "\"ChoiceImp5.Rdata\"", c("G", "Y"))
load("PinkDemogImp2-5.Rdata")
impClean(filled.pink.demog.dat.5, x.background,
         "PinkImp5", "\"PinkImp5.Rdata\"", c("P"))
load("BlueDemogImp2-5.Rdata")
impClean(filled.blue.demog.dat.5, x.background,
         "BlueImp5", "\"BlueImp5.Rdata\"", c("B"))







 
load("ChoiceDemogImp6.Rdata")
impClean(filled.choice.demog.dat.6, x.background,
         "ChoiceImp6", "\"ChoiceImp6.Rdata\"", c("G", "Y"))
load("PinkDemogImp2-6.Rdata")
impClean(filled.pink.demog.dat.6, x.background,
         "PinkImp6", "\"PinkImp6.Rdata\"", c("P"))
load("BlueDemogImp2-6.Rdata")
impClean(filled.blue.demog.dat.6, x.background,
         "BlueImp6", "\"BlueImp6.Rdata\"", c("B"))







 
load("ChoiceDemogImp7.Rdata")
impClean(filled.choice.demog.dat.7, x.background,
        "ChoiceImp7", "\"ChoiceImp7.Rdata\"", c("G", "Y"))
load("PinkDemogImp2-7.Rdata")
impClean(filled.pink.demog.dat.7, x.background,
         "PinkImp7", "\"PinkImp7.Rdata\"", c("P"))
load("BlueDemogImp2-7.Rdata")
impClean(filled.blue.demog.dat.7, x.background,
         "BlueImp7", "\"BlueImp7.Rdata\"", c("B"))







 
load("ChoiceDemogImp8.Rdata")
impClean(filled.choice.demog.dat.8, x.background,
         "ChoiceImp8", "\"ChoiceImp8.Rdata\"", c("G", "Y"))
load("PinkDemogImp2-8.Rdata")
impClean(filled.pink.demog.dat.8, x.background,
         "PinkImp8", "\"PinkImp8.Rdata\"", c("P"))
load("BlueDemogImp2-8.Rdata")
impClean(filled.blue.demog.dat.8, x.background,
         "BlueImp8", "\"BlueImp8.Rdata\"", c("B"))







 
load("ChoiceDemogImp9.Rdata")
impClean(filled.choice.demog.dat.9, x.background,
         "ChoiceImp9", "\"ChoiceImp9.Rdata\"", c("G", "Y"))
load("PinkDemogImp2-9.Rdata")
impClean(filled.pink.demog.dat.9, x.background,
         "PinkImp9", "\"PinkImp9.Rdata\"", c("P"))
load("BlueDemogImp2-9.Rdata")
impClean(filled.blue.demog.dat.9, x.background,
         "BlueImp9", "\"BlueImp9.Rdata\"", c("B"))







 
load("ChoiceDemogImp0.Rdata")
impClean(filled.choice.demog.dat.0, x.background,
         "ChoiceImp0", "\"ChoiceImp0.Rdata\"", c("G", "Y"))
load("PinkDemogImp2-0.Rdata")
impClean(filled.pink.demog.dat.0, x.background,
         "PinkImp0", "\"PinkImp0.Rdata\"", c("P"))
load("BlueDemogImp2-0.Rdata")
impClean(filled.blue.demog.dat.0, x.background,
         "BlueImp0", "\"BlueImp0.Rdata\"", c("B"))







