filepath <- "./"
filepathDataAnalysis <- "./"
NumOfImps <- 10

library(mitools)
library(survey)
library(Hmisc)
library(lattice)

MakeRegularList <- function(fpath, impname, nimps, choices){
  #  Function to create a list of imputations (a regular list, not an imputationList)
  #  fpath is the filepath in which the imputations are located.
  #  impname is the name given to each imputation, except for an index.
  #      The assumption is that the imputations have names impname0.Rdata,
  #      impname1.Rdata, impname2.Rdata, etc up to impname(nimps-1).Rdata.
  #  nimps is the number of multiple imputations.
  #  choices indicates whether we have choices or experience data.

  ImpList <- vector("list", nimps)  # space for list of dataframes

  #  Population ImpList with the imputations
  for(ii in 1:nimps){
    #  create the name, load the imputation
    Shrtname <- paste(impname, (ii-1), "Aug", sep = "")
        Impname <- paste(fpath, Shrtname, ".Rdata", sep = "")
    load(Impname)
    allforms.ii <- eval(parse(text = Shrtname))

    #  imputations have all (pink, blue, green, yellow) forms
    #      need to reduce only those relevant for analysis
    if(choices){
      locs.vec <- 1:(nrow(allforms.ii))
      locs.use <- locs.vec[allforms.ii$MiniColor == "G"]
      locs.use <- c(locs.use, locs.vec[allforms.ii$MiniColor == "Y"])
      useforms.ii <- allforms.ii[locs.use,]
    }
    if(!choices){
      locs.vec <- 1:(nrow(allforms.ii))
      locs.use <- c(locs.vec[allforms.ii$MiniColor == "B"],
                    locs.vec[allforms.ii$MiniColor == "P"])
      useforms.ii <- allforms.ii[locs.use,]
    }
    
    ImpList[[ii]] <- useforms.ii
  }

  #  Return regular list (NOT of class imputationList)
  ImpList
}


PPSvyResults <- function(imp.iL, varname){
  #  Takes imp.iL (an object of class imputationList) and varname (a string)
  #      varname should have only three levels:  "Yes", "No", and "NoVote"
  #      Returns simple random sample results as follows:
  #           (a)  "Yes" votes as a percentage of "Yes" and "No" votes
  #           (b)  "NoVote" as a percentage of all persons in the poll
  #      For each (a) and (b), returns PtEst, SE, CI95Low, CI95Up
  #  Appropriate values for varname:
  #      "IsOba2Party"
  #      "EndTax"
  #      "EndPotPen"
  #      "EndDogRace"
  as.num.script <- paste("as.numeric(", varname, " == ", sep = "")
  update.script <- paste("imp.iL <- update(imp.iL, YesVt = ",
                         as.num.script, "'Yes'), NoVt = ",
                         as.num.script, "'NoVote'), AnyVt = 1-",
                         as.num.script, "'NoVote'))", sep = "")
  eval(parse(text = update.script))
  clus.des.nofpc.PP <- svydesign(id=~1, probs =~Vtrfpc, data = imp.iL)
  YesVtPer.results.PP <- with(clus.des.nofpc.PP, svyratio(numerator=~ YesVt,
                                                        denominator=~ AnyVt,
                                                        covmat = TRUE))
  Yesrats.PP <- MIextract(YesVtPer.results.PP, fun = function(x) drop(coef(x)))
  Yesvars.PP <- MIextract(YesVtPer.results.PP, fun = vcov)
  Yesout.PP <- summary(MIcombine(Yesrats.PP, Yesvars.PP))
  
  NoVt.results.PP <- with(clus.des.nofpc.PP, svyratio(numerator=~NoVt,
                                                      denominator=~IsInPoll,
                                                      covmat = TRUE))
  Norats.PP <- MIextract(NoVt.results.PP,  fun = function(x) drop(coef(x)))
  Novars.PP <- MIextract(NoVt.results.PP, fun = vcov)
  Noout.PP <- summary(MIcombine(Norats.PP, Novars.PP))

  list("Yes.results" = Yesout.PP[1:4],
       "No.results" = Noout.PP[1:4])
}



Choice.RegList <- MakeRegularList(filepath, "ChoiceImp", NumOfImps, choices = T)
PPNames <- names(table(Choice.RegList[[1]]$MiniLoc))
NumPPs <- length(PPNames)

##  Construct the list that will hold the results
ResultsListByPP <- vector("list", 8)
names(ResultsListByPP) <- c("ObaPresObaMcC", "PresNoVt",
                            "YesTaxYesNo", "TaxNoVt",
                            "YesPotYesNo", "PotNoVt",
                            "YesDogYesNo", "DogNoVt")
ResultsMat <- matrix(NA, nrow = NumPPs, ncol = 6)
colnames(ResultsMat) <- c("PtEst", "SE", "CI95Low", "CI95Up",
                          "Truth", "PtEstmTruth")
rownames(ResultsMat) <- PPNames
for(ii in 1:8){
  ResultsListByPP[[ii]] <- ResultsMat
}

#  PPnum <- 4

for(PPnum in 1:NumPPs){
  ListForThisPP <- vector("list", NumOfImps)
  for(jj in 1:NumOfImps){
    tempdf <- Choice.RegList[[jj]]
    ListForThisPP[[jj]] <- tempdf[tempdf$MiniLoc == PPNames[PPnum],]
  }

  #  Obtain the estimates using the survey package
  ListForThisPP.iL <- imputationList(ListForThisPP)
  PresResults <- PPSvyResults(ListForThisPP.iL, "IsOba2Party")
  TaxResults <- PPSvyResults(ListForThisPP.iL, "EndTax")
  PotResults <- PPSvyResults(ListForThisPP.iL, "EndPotPen")
  DogResults <- PPSvyResults(ListForThisPP.iL, "EndDogRace")

  ResultsListByPP[["ObaPresObaMcC"]][PPNames[PPnum],1:4] <- as.numeric(PresResults[[1]])
  ResultsListByPP[["PresNoVt"]][PPNames[PPnum],1:4] <- as.numeric(PresResults[[2]])

  ResultsListByPP[["YesTaxYesNo"]][PPNames[PPnum],1:4] <- as.numeric(TaxResults[[1]])
  ResultsListByPP[["TaxNoVt"]][PPNames[PPnum],1:4] <- as.numeric(TaxResults[[2]])

  ResultsListByPP[["YesPotYesNo"]][PPNames[PPnum],1:4] <- as.numeric(PotResults[[1]])
  ResultsListByPP[["PotNoVt"]][PPNames[PPnum],1:4] <- as.numeric(PotResults[[2]])

  ResultsListByPP[["YesDogYesNo"]][PPNames[PPnum],1:4] <- as.numeric(DogResults[[1]])
  ResultsListByPP[["DogNoVt"]][PPNames[PPnum],1:4] <- as.numeric(DogResults[[2]])

  #  Obtain the truth, this is a bigger pain in the ass
  RD <- ListForThisPP[[1]]  # any imputation has same values of the truth, "RD" for "Reduced Data"
  BallotsCast <- unique(RD[, "BallotsCast"])
  ResultsListByPP[["ObaPresObaMcC"]][PPNames[PPnum],5] <- unique(RD[,"PresObama"])/unique(RD[,"PresObama"]+
                                                                                         RD[,"PresMcCain"])
  ResultsListByPP[["PresNoVt"]][PPNames[PPnum],5] <- unique(RD[,"PresBlank"])/BallotsCast

  ResultsListByPP[["YesTaxYesNo"]][PPNames[PPnum],5] <- unique(RD[,"EndTaxYes"])/unique(RD[,"EndTaxYes"] +
                                                                             RD[,"EndTaxNo"])
  ResultsListByPP[["TaxNoVt"]][PPNames[PPnum],5] <- unique(RD[,"EndTaxBlank"])/BallotsCast

  ResultsListByPP[["YesPotYesNo"]][PPNames[PPnum],5] <-unique(RD[,"EndPotPenYes"])/unique(RD[,"EndPotPenYes"] +
                                                                             RD[,"EndPotPenNo"])
  ResultsListByPP[["PotNoVt"]][PPNames[PPnum],5] <- unique(RD[,"EndPotPenBlanks"])/BallotsCast

  ResultsListByPP[["YesDogYesNo"]][PPNames[PPnum],5] <- unique(RD[,"EndDogRaceYes"])/unique(RD[,"EndDogRaceYes"] +
                                                                             RD[,"EndDogRaceNo"])
  ResultsListByPP[["DogNoVt"]][PPNames[PPnum],5] <- unique(RD[,"EndDogRaceBlanks"])/BallotsCast
}


for(ii in 1:8){
  ResultsListByPP[[ii]][,"PtEstmTruth"] <- ResultsListByPP[[ii]][,"PtEst"] - ResultsListByPP[[ii]][,"Truth"]
  for(jj in 1:nrow(ResultsListByPP[[ii]])){
    ResultsListByPP[[ii]][jj,"CI95Low"] <- max(0, ResultsListByPP[[ii]][jj,"CI95Low"])
    ResultsListByPP[[ii]][jj,"CI95Up"] <- min(1, ResultsListByPP[[ii]][jj,"CI95Up"])
  }
  ResultsListByPP[[ii]] <- as.data.frame(ResultsListByPP[[ii]])  #  need to turn these into dataframes now
                                                                 #  so that later incorporation of PP names
                                                                 #  does not turn the numbers into factors
}










## two-party pres vote
ResultsListByPPNames <- names(ResultsListByPP)
ShrtNames <- c("AnnAp", "AreaE", "BHill",
               "BeetS", "BMidS", "BTriA",
               "BosUn", "CatHS", "CodSq",
               "CurHa", "EaLnC", "ElGrS",
               "FloHl", "FolBd", "GaviS",
               "GeCoS", "HoAlL", "GarfS",
               "ChitS", "KentS", "LoMiL",
               "LuStS", "MaGaG", "McKiS",
               "Metro", "MozaS", "OrGaS",
               "PaOHS", "PhBaS", "RoBrL",
               "StAnS", "ShelC", "StMaS",
               "ThEdS", "ThKeS", "ToMuB",
               "UpCoH", "TaftS", "WoWiS")
ShrtNames <- paste("Location", 1:39, sep="")
PolLoc <- factor(ShrtNames, levels=ShrtNames)
Diagnosticgraphfilename <- paste(filepathDataAnalysis, "TwoPartyPresByPollingLoc.pdf", sep = "")
#pdf(file = Diagnosticgraphfilename, paper = "a4r", onefile = T)
pdf(file = Diagnosticgraphfilename, onefile = T)

ii <- 1
mainName <- ResultsListByPPNames[ii]
tempdata <- ResultsListByPP[[ii]]
tempdata <- cbind(PolLoc, tempdata)
print(Dotplot(PolLoc~Cbind(Truth, CI95Low, CI95Up), data = tempdata,
        main = "Obama Fraction of Two-Party Vote", xlab = "Truth & 95%CI", ylab="Polling Location"))
dev.off()
