library(MCMCpack)
library(splines)



neighbors <- function(x, window){
  nbrs <- rep(NA, length(x))
  for (i in 1:length(x)){
    nbrs[i] <- sum(x > x[i] - window & x <= x[i] + window)
  }
  return(nbrs)
}



inds <- 0:9

for (i in inds){
  pinkstring <- paste("./PinkImp", i, ".csv", sep="")
  bluestring <- paste("./BlueImp", i, ".csv", sep="")
  
  pink <- read.csv(pinkstring, stringsAsFactors=FALSE)
  blue <- read.csv(bluestring, stringsAsFactors=FALSE) 

  ## locations whose Obama vote % not coverered by 95% CI
  drop.loc <- c( "Saint Anne's School",
                "Shelburne Center",
                "Orchard Gardens Elementary School",
                "Florian Hall",
                "James J. Chittick School",
                "Beethoven School")

  ## Woodbourne Apartments not here "Marcus Garvey Gardens" replaces it
  treated.loc <- c("Gavin School",
                   "Boston Middle School Academy (Formally Endicott Elementary School)",
                   "Mozart School",
                   "Saint Anne's School",
                   "James A. Garfield School",
                   "St. Mark's School Hall",
                   "Roslindale Branch Library",
                   "Boston Trinity Academy ",
                   "Beethoven School",
                   "Curtis Hall" ,
                   "James J. Chittick School",
                   "Honan-Allston Branch Library",
                   "Marcus Garvey Gardens")
  
  ## Agassiz School not here "Shelburne Center"  replaces it
  control.loc <- c("Thomas A. Edison School",
                   "William Howard Taft School" ,
                   "Patrick O'Hearn School",
                   "Phineas Bates School",
                   "Lower Mills Library",
                   "George H. Conley School",
                   "Foley Building",
                   "McKinley School",
                   "Early Learning Center" ,
                   "Woodrow Wilson School",
                   "Area E Police Station" ,
                   "Thomas J Kenny School",
                   "Shelburne Center" )
  
  
  
  pink.sub <- pink[,c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                      "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")]
  blue.sub <- blue[,c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                      "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")]
  colnames(pink.sub) <- c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                          "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")
  colnames(blue.sub) <- c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                          "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")
  
  
  
  pink.sub$treated <- pink.sub$MiniLoc %in% treated.loc
  pink.sub$control <- pink.sub$MiniLoc %in% control.loc
  pink.sub$notmatched <- !(pink.sub$MiniLoc %in% c(control.loc, treated.loc))
  pink.sub$pink <- 1
  pink.sub$SIDnum <- 1
  pink.sub$SIDnum[pink.sub$ShowID == "No"] <- 0
  pink.sub$IsVtrFem <- pink.sub$IsVtrFem == "Yes"
  pink.sub$IsVtrHisp <- pink.sub$IsVtrHisp == "Yes"
  pink.sub$IsVtrAfrAm <- pink.sub$VtrRace == "Black"
  pink.sub$IsVtrAsian <- pink.sub$VtrRace == "Asian"
  pink.sub$NonEngSpkr <- pink.sub$VtrHmLang != "Engl"
  pink.sub$CollegePlus <- pink.sub$Educ == "4YrColl" | pink.sub$Educ == "PostGrad"
  pink.sub$Young <- pink.sub$VtrAge == "<=25"
  pink.sub$Old <- pink.sub$VtrAge == ">55" 
  
  blue.sub$treated <- blue.sub$MiniLoc %in% treated.loc
  blue.sub$control <- blue.sub$MiniLoc %in% control.loc
  blue.sub$notmatched <- !(blue.sub$MiniLoc %in% c(control.loc, treated.loc))
  blue.sub$pink <- 0
  blue.sub$SIDnum <- 1
  blue.sub$SIDnum[blue.sub$ShowID == "No"] <- 0
  blue.sub$IsVtrFem <- blue.sub$IsVtrFem == "Yes"
  blue.sub$IsVtrHisp <- blue.sub$IsVtrHisp == "Yes"
  blue.sub$IsVtrAfrAm <- blue.sub$VtrRace == "Black"
  blue.sub$IsVtrAsian <- blue.sub$VtrRace == "Asian"
  blue.sub$NonEngSpkr <- blue.sub$VtrHmLang != "Engl"
  blue.sub$CollegePlus <- blue.sub$Educ == "4YrColl" | blue.sub$Educ == "PostGrad"
  blue.sub$Young <- blue.sub$VtrAge == "<=25"
  blue.sub$Old <- blue.sub$VtrAge == ">55" 
  
  mydata <- rbind(pink.sub, blue.sub)
  #mydata <- mydata[!mydata$notmatched,]
  drop.indic <- mydata$IsVtrHisp == FALSE & mydata$VtrRace == "OthRace"
  mydata <- mydata[!drop.indic,]
  mydata <- mydata[!is.na(mydata$MiniTime),]
  ## drop the polling locations where Obama vote % not covered by 95% CI
  mydata <- mydata[!(mydata$MiniLoc %in% drop.loc),]

  mydata$congestion <- NA
  locations <- unique(mydata$MiniLoc)

  for (loc in locations){
    mydata$congestion[mydata$MiniLoc == loc] <- neighbors(mydata$MiniTime[mydata$MiniLoc == loc], 30) / sum(mydata$MiniLoc == loc)
  }

  cong.spline.mat <- bs(mydata$congestion, knots=c(0.075, 0.125), degree=3,
                        Boundary.knots=c(0, 0.3))[,]
  
  y <- mydata$SIDnum
  X <- cbind(1, mydata$treated, mydata$pink, mydata$IsVtrFem, mydata$IsVtrHisp,
             mydata$IsVtrAfrAm, mydata$IsVtrAsian, mydata$NonEngSpkr,
             mydata$CollegePlus, mydata$Young, mydata$Old,
             cong.spline.mat)
  colnames(X) <- c("Intercept", "Treated", "Pink", "Female", "Hisp", "AfrAm",
                   "Asian", "NonEngSpkr", "CollegePlus", "Young", "Old",
                   "cong1", "cong2", "cong3", "cong4", "cong5")
                                        #W <- matrix(1, nrow(X), 1)
  W <- X[,c(1), drop=FALSE]
  n <- length(unique(mydata$MiniLoc))
  
  
  
  Wtilde <- matrix(0, nrow(W), n*ncol(W))
  counter <- 1
  m <- NULL
  for (loc.name in unique(mydata$MiniLoc)){
    Wtilde[mydata$MiniLoc == loc.name, counter:(counter+ncol(W)-1)] <-
      W[mydata$MiniLoc == loc.name,]
    counter <- counter + ncol(W)
    m <- c(m, sum(mydata$MiniLoc == loc.name))
  }
  
  
  mydata$cong1 <- cong.spline.mat[,1]
  mydata$cong2 <- cong.spline.mat[,2]
  mydata$cong3 <- cong.spline.mat[,3]
  mydata$cong4 <- cong.spline.mat[,4]
  mydata$cong5 <- cong.spline.mat[,5]
  mydata$MiniLocNum <- as.numeric(as.factor(mydata$MiniLoc))
  
  
  #out.basic <- MCMChieroprobit(y ~ 0+X, fullW=W, Wtilde=Wtilde, 
  #                             m=m, b0=0, B0=0.000001, tune=4.5, beta.start=0,
  #                             r0=3, R0=diag(.3, ncol(W)),
  #                             thin=50, burnin=5000, mcmc=200000)
  
  out.fixed <- glm(SIDnum ~ as.factor(MiniLocNum) + pink +
                   IsVtrFem + IsVtrHisp + IsVtrAfrAm +
                   IsVtrAsian + NonEngSpkr + CollegePlus + Young + Old +
                   cong1 + cong2 + cong3 + cong4 + cong5,
                   data=mydata, family=binomial(probit))
  
  cat("Imputation ", i, "\n")

  beta <- coef(out.fixed)
  se <- sqrt(diag(vcov(out.fixed)))
  #ci <- confint(out.fixed)
  #ci.lower <- ci[,1]
  #ci.upper <- ci[,2]
  
  eval(parse(text=paste("beta.", i, " <- beta", sep="")))
  eval(parse(text=paste("se.", i, " <- se", sep="")))
  #eval(parse(text=paste("ci.lower.", i, " <- ci.lower", sep="")))
  #eval(parse(text=paste("ci.upper.", i, " <- ci.upper", sep="")))
  
}

beta <- rbind(beta.0,
              beta.1,
              beta.2,
              beta.3,
              beta.4,
              beta.5,
              beta.6,
              beta.7,
              beta.8,
              beta.9)

se <- rbind(se.0,
            se.1,
            se.2,
            se.3,
            se.4,
            se.5,
            se.6,
            se.7,
            se.8,
            se.9)

#ci.lower <- rbind(ci.lower.0,
#              ci.lower.1,
#              ci.lower.2,
#              ci.lower.3,
#              ci.lower.4,
#              ci.lower.5,
#              ci.lower.6,
#              ci.lower.7,
#              ci.lower.8,
#              ci.lower.9)

#ci.upper <- rbind(ci.upper.0,
#              ci.upper.1,
#              ci.upper.2,
#              ci.upper.3,
#              ci.upper.4,
#              ci.upper.5,
#              ci.upper.6,
#              ci.upper.7,
#              ci.upper.8,
#              ci.upper.9)


save.image("fixed-drop1.Rdata")
