library(MCMCpack)
library(splines)



## raw (non-imputed) data 
raw <- read.csv("./CombinedData.csv", stringsAsFactors=FALSE)

raw$VtrRace = NA
raw$VtrRace[raw$IsVtrAsian == 1] <- "Asian"   ## Asian
raw$VtrRace[raw$IsVtrBlack == 1] <- "Black"   ## Black
raw$VtrRace[raw$IsVtrWhite == 1] <- "White"   ## White
raw$VtrRace[raw$IsVtrOthRace == 1] <- "Other" ## OthRace


raw$ShowID2 <- NA
raw$ShowID2[raw$ShowID == "NO"] <- 0
raw$ShowID2[raw$ShowID == "No"] <- 0
raw$ShowID2[raw$ShowID == "Yes"] <- 1
raw$ShowID2[raw$ShowID == "NoMemory"] <- NA
raw$ShowID <- raw$ShowID2

raw$VtrAge <- 2008 - raw$VtrYrBorn
raw$VtrAge[raw$VtrAge < 18] <- NA
raw$VtrAge[raw$VtrAge <= 25] <- 1
raw$VtrAge[raw$VtrAge > 25 & raw$VtrAge <= 35] <- 2
raw$VtrAge[raw$VtrAge > 35 & raw$VtrAge <= 45] <- 3
raw$VtrAge[raw$VtrAge > 45 & raw$VtrAge <= 55] <- 4
raw$VtrAge[raw$VtrAge > 55] <- 5



neighbors <- function(x, window){
  nbrs <- rep(NA, length(x))
  for (i in 1:length(x)){
    nbrs[i] <- sum(x > x[i] - window & x <= x[i] + window)
  }
  return(nbrs)
}

## Woodbourne Apartments not here "Marcus Garvey Gardens" replaces it
treated.loc <- c("Gavin School",
                 "Boston Middle School Academy (Formally Endicott Elementary School)",
                 "Mozart School",
                 "Saint Anne's School",
                 "James A. Garfield School",
                 "St. Mark's School Hall",
                 "Roslindale Branch Library",
                 "Boston Trinity Academy ",
                 "Beethoven School",
                 "Curtis Hall" ,
                 "James J. Chittick School",
                 "Honan-Allston Branch Library",
                 "Marcus Garvey Gardens")

## Agassiz School not here "Shelburne Center"  replaces it
control.loc <- c("Thomas A. Edison School",
                 "William Howard Taft School" ,
                 "Patrick O'Hearn School",
                 "Phineas Bates School",
                 "Lower Mills Library",
                 "George H. Conley School",
                 "Foley Building",
                 "McKinley School",
                 "Early Learning Center" ,
                 "Woodrow Wilson School",
                 "Area E Police Station" ,
                 "Thomas J Kenny School",
                 "Shelburne Center" )



 






raw.sub <- raw[,c("ShowID", "MiniLoc", "MiniColor", "IsVtrFem", "IsVtrHisp",
                    "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime",
                  "IsPstrWhite",
                  "TotalVAP00", "WhiteVAP00", "BlackVAP00", "HispanicVAP00",
                  "AsianVAP00")]

colnames(raw.sub) <- c("ShowID", "MiniLoc", "MiniColor",
                       "IsVtrFem", "IsVtrHisp",
                        "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime",
                       "IsPstrWhite",
                       "tVAP", "wVAP", "bVAP", "hVAP", "aVAP")




raw.sub$bVAPpct <- raw.sub$bVAP / raw.sub$tVAP
raw.sub$hVAPpct <- raw.sub$hVAP / raw.sub$tVAP
raw.sub$wVAPpct <- raw.sub$wVAP / raw.sub$tVAP
raw.sub$aVAPpct <- raw.sub$aVAP / raw.sub$tVAP

raw.sub$pink <- raw.sub$MiniColor == "P"  
raw.sub$treated <- raw.sub$MiniLoc %in% treated.loc
raw.sub$control <- raw.sub$MiniLoc %in% control.loc
raw.sub$notmatched <- !(raw.sub$MiniLoc %in% c(control.loc, treated.loc))
raw.sub$SIDnum <- raw.sub$ShowID
raw.sub$IsVtrAfrAm <- raw.sub$VtrRace == "Black"
raw.sub$IsVtrAsian <- raw.sub$VtrRace == "Asian"
raw.sub$NonEngSpkr <- raw.sub$VtrHmLang != "Engl"
raw.sub$CollegePlus <- raw.sub$Educ == 4 | raw.sub$Educ == 5
raw.sub$Young <- raw.sub$VtrAge == 1
raw.sub$Old <- raw.sub$VtrAge == 5 

mydata <-  raw.sub

holder <- strsplit(mydata$MiniTime, split=":")
for (i in 1:length(holder)){
  if (length(holder[[i]] > 1)){
    mydata$MiniTime[i] <- as.numeric(holder[[i]][1])*60 +
      as.numeric(holder[[i]][2])
  }
  cat("** ", i, "\n")
}
mydata$MiniTime <- as.numeric(mydata$MiniTime)

mydata <- na.omit(mydata)


drop.indic <- mydata$IsVtrHisp == 0 & mydata$VtrRace == "OthRace"
mydata <- subset(mydata, subset=!drop.indic)
##mydata <- mydata[!drop.indic,]
mydata <- mydata[!is.na(mydata$MiniTime),]





mydata$congestion <- NA
locations <- unique(mydata$MiniLoc)

for (loc in locations){
  mydata$congestion[mydata$MiniLoc == loc] <- neighbors(mydata$MiniTime[mydata$MiniLoc == loc], 30) / sum(mydata$MiniLoc == loc)
}

cong.spline.mat <- bs(mydata$congestion, knots=c(0.075, 0.125), degree=3,
                      Boundary.knots=c(0, 0.5))[,]



y <- mydata$SIDnum
X <- cbind(1, mydata$treated, mydata$pink, mydata$IsVtrFem, mydata$IsVtrHisp,
           mydata$IsVtrAfrAm, mydata$IsVtrAsian, mydata$NonEngSpkr,
           mydata$CollegePlus, mydata$Young, mydata$Old,
           cong.spline.mat[,1], cong.spline.mat[,2], cong.spline.mat[,3],
           cong.spline.mat[,4], cong.spline.mat[,5],
           mydata$treated*mydata$pink, mydata$treated*mydata$IsVtrFem,
           mydata$treated*mydata$IsVtrHisp,
           mydata$treated*mydata$IsVtrAfrAm,
           mydata$treated*mydata$IsVtrAsian,
           mydata$treated*mydata$NonEngSpkr,
           mydata$treated*mydata$CollegePlus,
           mydata$treated*mydata$Young,
           mydata$treated*mydata$Old,
           mydata$treated*cong.spline.mat[,1],
           mydata$treated*cong.spline.mat[,2],
           mydata$treated*cong.spline.mat[,3],
           mydata$treated*cong.spline.mat[,4],
           mydata$treated*cong.spline.mat[,5],
           mydata$bVAPpct, mydata$hVAPpct, mydata$aVAPpct,
           mydata$IsPstrWhite,
           mydata$IsVtrHisp*mydata$IsPstrWhite,
           mydata$IsVtrAfrAm*mydata$IsPstrWhite,
           mydata$IsVtrAsian*mydata$IsPstrWhite,
           mydata$NonEngSpkr*mydata$IsPstrWhite)
colnames(X) <- c("Intercept", "Treated", "Pink", "Female", "Hisp", "AfrAm",
                 "Asian", "NonEngSpkr", "CollegePlus", "Young", "Old",
                 "cong1", "cong2", "cong3", "cong4", "cong5",
                 "Tr-Pink", "Tr-Female", "Tr-Hisp", "Tr-AfrAm",
                 "Tr-Asian", "Tr-NonEngSpkr", "Tr-CollegePlus",
                 "Tr-Young", "Tr-Old",
                 "Tr-cong1", "Tr-cong2", "Tr-cong3", "Tr-cong4", "Tr-cong5",
                 "bVAPpct", "hVAPpct", "aVAPpct",
                 "IsPstrWhite", "Hisp-PstrWhite",
                 "AfrAm-PstrWhite","Asian-PstrWhite",
                 "NonEngSpkr-PstrWhite")
                                        #W <- matrix(1, nrow(X), 1)
W <- X[,c(1,5,6,7)]
n <- length(unique(mydata$MiniLoc))



Wtilde <- matrix(0, nrow(W), n*ncol(W))
counter <- 1
m <- NULL
for (loc.name in unique(mydata$MiniLoc)){
  Wtilde[mydata$MiniLoc == loc.name, counter:(counter+ncol(W)-1)] <-
    W[mydata$MiniLoc == loc.name,]
  counter <- counter + ncol(W)
  m <- c(m, sum(mydata$MiniLoc == loc.name))
}



out.basic <- MCMChieroprobit(y ~ 0+X, fullW=W, Wtilde=Wtilde, 
                             m=m, b0=0, B0=0.000001, tune=4.5, beta.start=0,
                             r0=6, R0=diag(.6, ncol(W)),
                             thin=50, burnin=5000, mcmc=200000)



print( t(round(apply(out.basic, 2, quantile, c(0.025, 0.5, 0.975)),3)) )


save.image("hier16-raw.Rdata")
