library(MCMCpack)
library(splines)

neighbors <- function(x, window){
  nbrs <- rep(NA, length(x))
  for (i in 1:length(x)){
    nbrs[i] <- sum(x > x[i] - window & x <= x[i] + window)
  }
  return(nbrs)
}



inds <- 0:9

for (i in inds){
  pinkstring <- paste("./PinkImp", i, ".csv", sep="")
  bluestring <- paste("./BlueImp", i, ".csv", sep="")
  
  pink <- read.csv(pinkstring, stringsAsFactors=FALSE)
  blue <- read.csv(bluestring, stringsAsFactors=FALSE) 
  
  ## Woodbourne Apartments not here "Marcus Garvey Gardens" replaces it
  treated.loc <- c("Gavin School",
                   "Boston Middle School Academy (Formally Endicott Elementary School)",
                   "Mozart School",
                   "Saint Anne's School",
                   "James A. Garfield School",
                   "St. Mark's School Hall",
                   "Roslindale Branch Library",
                   "Boston Trinity Academy ",
                   "Beethoven School",
                   "Curtis Hall" ,
                   "James J. Chittick School",
                   "Honan-Allston Branch Library",
                   "Marcus Garvey Gardens")
  
  ## Agassiz School not here "Shelburne Center"  replaces it
  control.loc <- c("Thomas A. Edison School",
                   "William Howard Taft School" ,
                   "Patrick O'Hearn School",
                   "Phineas Bates School",
                   "Lower Mills Library",
                   "George H. Conley School",
                   "Foley Building",
                   "McKinley School",
                   "Early Learning Center" ,
                   "Woodrow Wilson School",
                   "Area E Police Station" ,
                   "Thomas J Kenny School",
                   "Shelburne Center" )
  
  
  
  pink.sub <- pink[,c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                      "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")]
  blue.sub <- blue[,c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                      "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")]
  colnames(pink.sub) <- c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                          "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")
  colnames(blue.sub) <- c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                          "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")
  
  
  
  pink.sub$treated <- pink.sub$MiniLoc %in% treated.loc
  pink.sub$control <- pink.sub$MiniLoc %in% control.loc
  pink.sub$notmatched <- !(pink.sub$MiniLoc %in% c(control.loc, treated.loc))
  pink.sub$pink <- 1
  pink.sub$SIDnum <- 1
  pink.sub$SIDnum[pink.sub$ShowID == "No"] <- 0
  pink.sub$IsVtrFem <- pink.sub$IsVtrFem == "Yes"
  pink.sub$IsVtrHisp <- pink.sub$IsVtrHisp == "Yes"
  pink.sub$IsVtrAfrAm <- pink.sub$VtrRace == "Black"
  pink.sub$IsVtrAsian <- pink.sub$VtrRace == "Asian"
  pink.sub$NonEngSpkr <- pink.sub$VtrHmLang != "Engl"
  pink.sub$CollegePlus <- pink.sub$Educ == "4YrColl" | pink.sub$Educ == "PostGrad"
  pink.sub$Young <- pink.sub$VtrAge == "<=25"
  pink.sub$Old <- pink.sub$VtrAge == ">55" 
  
  blue.sub$treated <- blue.sub$MiniLoc %in% treated.loc
  blue.sub$control <- blue.sub$MiniLoc %in% control.loc
  blue.sub$notmatched <- !(blue.sub$MiniLoc %in% c(control.loc, treated.loc))
  blue.sub$pink <- 0
  blue.sub$SIDnum <- 1
  blue.sub$SIDnum[blue.sub$ShowID == "No"] <- 0
  blue.sub$IsVtrFem <- blue.sub$IsVtrFem == "Yes"
  blue.sub$IsVtrHisp <- blue.sub$IsVtrHisp == "Yes"
  blue.sub$IsVtrAfrAm <- blue.sub$VtrRace == "Black"
  blue.sub$IsVtrAsian <- blue.sub$VtrRace == "Asian"
  blue.sub$NonEngSpkr <- blue.sub$VtrHmLang != "Engl"
  blue.sub$CollegePlus <- blue.sub$Educ == "4YrColl" | blue.sub$Educ == "PostGrad"
  blue.sub$Young <- blue.sub$VtrAge == "<=25"
  blue.sub$Old <- blue.sub$VtrAge == ">55" 
  
  mydata <- rbind(pink.sub, blue.sub)
  drop.indic <- mydata$IsVtrHisp == FALSE & mydata$VtrRace == "OthRace"
  mydata <- mydata[!drop.indic,]
  mydata <- mydata[!is.na(mydata$MiniTime),]

  mydata$congestion <- NA
  locations <- unique(mydata$MiniLoc)

  for (loc in locations){
    mydata$congestion[mydata$MiniLoc == loc] <- neighbors(mydata$MiniTime[mydata$MiniLoc == loc], 30) / sum(mydata$MiniLoc == loc)
  }

  cong.spline.mat <- bs(mydata$congestion, knots=c(0.075, 0.125), degree=3,
                        Boundary.knots=c(0, 0.3))[,]
  
  y <- mydata$SIDnum
  X <- cbind(1, mydata$treated, mydata$pink, mydata$IsVtrFem, mydata$IsVtrHisp,
             mydata$IsVtrAfrAm, mydata$IsVtrAsian, mydata$NonEngSpkr,
             mydata$CollegePlus, mydata$Young, mydata$Old,
             cong.spline.mat)
  colnames(X) <- c("Intercept", "Treated", "Pink", "Female", "Hisp", "AfrAm",
                   "Asian", "NonEngSpkr", "CollegePlus", "Young", "Old",
                   "cong1", "cong2", "cong3", "cong4", "cong5")
  W <- X[,c(1,5,6,7)]
  n <- length(unique(mydata$MiniLoc))
  
  
  
  Wtilde <- matrix(0, nrow(W), n*ncol(W))
  counter <- 1
  m <- NULL
  for (loc.name in unique(mydata$MiniLoc)){
    Wtilde[mydata$MiniLoc == loc.name, counter:(counter+ncol(W)-1)] <-
      W[mydata$MiniLoc == loc.name,]
    counter <- counter + ncol(W)
    m <- c(m, sum(mydata$MiniLoc == loc.name))
  }
  
  
  
  out.basic <- MCMChieroprobit(y ~ 0+X, fullW=W, Wtilde=Wtilde, 
                         m=m, b0=0, B0=0.000001, tune=4.5, beta.start=0,
                         r0=6, R0=diag(.6, ncol(W)),
                         thin=50, burnin=5000, mcmc=200000)

  cat("Imputation ", i, "\n")

  eval(parse(text=paste("out.basic.", i, " <- out.basic", sep="")))
  
}

out.basic <- rbind(out.basic.0,
                   out.basic.1,
                   out.basic.2,
                   out.basic.3,
                   out.basic.4,
                   out.basic.5,
                   out.basic.6,
                   out.basic.7,
                   out.basic.8,
                   out.basic.9)
                 
out.basic <- mcmc(out.basic, thin=50)


print( t(round(apply(out.basic, 2, quantile, c(0.025, 0.5, 0.975)),3)) )


save.image("hier2.Rdata")
