library(MCMCpack)

inds <- 0:9

pred.sum <- list()
pred.sum.overall <- 0

cong.sum <- list()
cong.sum.overall <- 0

ask.cong.sum <- list()
ask.cong.sum.overall <- 0

cong.points <- seq(from=0, to=.25, by=.00005)
F.sum.cong <- cong.points * 0




neighbors <- function(x, window){
  nbrs <- rep(NA, length(x))
  for (i in 1:length(x)){
    nbrs[i] <- sum(x > x[i] - window & x <= x[i] + window)
  }
  return(nbrs)
}


F2f <- function(F, xpoints, width=5){
  fvals <-  rep(NA, length(xpoints)-1)
  xvals <- rep(NA, length(xpoints)-1)
  i <- width
  while (i <= length(F)){
    area.i <- F[i] - F[i-width+1]
    width.i <- xpoints[i] - xpoints[i-width+1]
    fvals[i] <- area.i / width.i
    xvals[i] <- xpoints[i-width+1] + width.i
    i <- i + width
  }
  fvals <- na.omit(fvals)
  xvals <- na.omit(xvals)
  return (list(xvals=xvals, fvals=fvals))
}


for (i in inds){
  cong.long <- NULL
  minutes.long <- NULL
  showID.long <- NULL
  
  pinkstring <- paste("./PinkImp", i, ".csv", sep="")
  bluestring <- paste("./BlueImp", i, ".csv", sep="")
  
  pink <- read.csv(pinkstring, stringsAsFactors=FALSE)
  blue <- read.csv(bluestring, stringsAsFactors=FALSE) 
  
  ## Woodbourne Apartments not here "Marcus Garvey Gardens" replaces it
  treated.loc <- c("Gavin School",
                   "Boston Middle School Academy (Formally Endicott Elementary School)",
                   "Mozart School",
                   "Saint Anne's School",
                   "James A. Garfield School",
                   "St. Mark's School Hall",
                   "Roslindale Branch Library",
                   "Boston Trinity Academy ",
                   "Beethoven School",
                   "Curtis Hall" ,
                   "James J. Chittick School",
                   "Honan-Allston Branch Library",
                   "Marcus Garvey Gardens")
  
  ## Agassiz School not here "Shelburne Center"  replaces it
  control.loc <- c("Thomas A. Edison School",
                   "William Howard Taft School" ,
                   "Patrick O'Hearn School",
                   "Phineas Bates School",
                   "Lower Mills Library",
                   "George H. Conley School",
                   "Foley Building",
                   "McKinley School",
                   "Early Learning Center" ,
                   "Woodrow Wilson School",
                   "Area E Police Station" ,
                   "Thomas J Kenny School",
                   "Shelburne Center" )
  
  
  
  pink.sub <- pink[,c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                      "VtrRace", "Educ", "VtrAge", "MiniTime")]
  blue.sub <- blue[,c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                      "VtrRace", "Educ", "VtrAge", "MiniTime")]
  colnames(pink.sub) <- c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                          "VtrRace", "Educ", "VtrAge", "MiniTime")
  colnames(blue.sub) <- c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                          "VtrRace", "Educ", "VtrAge", "MiniTime")
  
  
  
  pink.sub$treated <- pink.sub$MiniLoc %in% treated.loc
  pink.sub$control <- pink.sub$MiniLoc %in% control.loc
  pink.sub$notmatched <- !(pink.sub$MiniLoc %in% c(control.loc, treated.loc))
  pink.sub$pink <- 1
  pink.sub$SIDnum <- 1
  pink.sub$SIDnum[pink.sub$ShowID == "No"] <- 0
  pink.sub$IsVtrFem <- pink.sub$IsVtrFem == "Yes"
  pink.sub$IsVtrHisp <- pink.sub$IsVtrHisp == "Yes"
  pink.sub$IsVtrAfrAm <- pink.sub$VtrRace == "Black"
  pink.sub$IsVtrAsian <- pink.sub$VtrRace == "Asian"
  pink.sub$CollegePlus <- pink.sub$Educ == "4YrColl" | pink.sub$Educ == "PostGrad"
  pink.sub$Young <- pink.sub$VtrAge == "<=25"
  pink.sub$Old <- pink.sub$VtrAge == ">55" 
  
  blue.sub$treated <- blue.sub$MiniLoc %in% treated.loc
  blue.sub$control <- blue.sub$MiniLoc %in% control.loc
  blue.sub$notmatched <- !(blue.sub$MiniLoc %in% c(control.loc, treated.loc))
  blue.sub$pink <- 0
  blue.sub$SIDnum <- 1
  blue.sub$SIDnum[blue.sub$ShowID == "No"] <- 0
  blue.sub$IsVtrFem <- blue.sub$IsVtrFem == "Yes"
  blue.sub$IsVtrHisp <- blue.sub$IsVtrHisp == "Yes"
  blue.sub$IsVtrAfrAm <- blue.sub$VtrRace == "Black"
  blue.sub$IsVtrAsian <- blue.sub$VtrRace == "Asian"
  blue.sub$CollegePlus <- blue.sub$Educ == "4YrColl" | blue.sub$Educ == "PostGrad"
  blue.sub$Young <- blue.sub$VtrAge == "<=25"
  blue.sub$Old <- blue.sub$VtrAge == ">55" 
  
  mydata <- rbind(pink.sub, blue.sub)
  drop.indic <- mydata$IsVtrHisp == FALSE & mydata$VtrRace == "OthRace"
  mydata <- mydata[!drop.indic,]
  mydata <- mydata[!is.na(mydata$MiniTime),]
  
  y <- mydata$SIDnum
  X <- cbind(1, mydata$treated, mydata$pink, mydata$IsVtrFem, mydata$IsVtrHisp,
             mydata$IsVtrAfrAm, mydata$IsVtrAsian, mydata$CollegePlus,
             mydata$Young, mydata$Old)
  colnames(X) <- c("Intercept", "Treated", "Pink", "Female", "Hisp", "AfrAm",
                   "Asian", "CollegePlus", "Young", "Old")


  locations <- unique(mydata$MiniLoc)
  
  cat("Imputation ", i, "\n")

  y.o <- mydata$SIDnum
  x.o <- mydata$MiniTime
  lo.out <- loess(y.o ~ x.o, span=.25, degree=1)
  xx <- seq(from=420, to=1200, by=5)
  pred.sum.overall <- pred.sum.overall + predict(lo.out, newdata=xx)
  
  for (loc in locations){
    
    y <- mydata$SIDnum[mydata$MiniLoc == loc]
    x <- mydata$MiniTime[mydata$MiniLoc == loc]
    cong <- neighbors(x, 30) / length(x)
    showID.long <- c(showID.long, y)
    cong.long <- c(cong.long, cong)
    minutes.long <- c(minutes.long, x)
    
    lo.out <- loess(y~x, span=.333, degree=1)
    cong.lo.out <- loess(cong ~ x, span=.2, degree=1)
    ask.cong.lo.out <- loess(y ~ cong, span=.85, degree=1)
    
    xx <- seq(from=420, to=1200, by=5)
    cc <- seq(from=0, to=0.25, by=.001)
    if (i == 0){
      pred.sum[[loc]] <- rep(0, length(xx))
      cong.sum[[loc]] <- rep(0, length(xx))
      ask.cong.sum[[loc]] <- rep(0, length(cc))
    }
    pred.sum[[loc]] <- pred.sum[[loc]] +  predict(lo.out, newdata=xx)
    cong.sum[[loc]] <- cong.sum[[loc]] + predict(cong.lo.out, newdata=xx)
    ask.cong.sum[[loc]] <- ask.cong.sum[[loc]] + predict(ask.cong.lo.out,
                                                         newdata=cc)
  }

  lo.out <- loess(showID.long  ~ cong.long, span=.5, degree=1)
  cc <- seq(from=0, to=0.25, by=.001)
  ask.cong.sum.overall <- ask.cong.sum.overall + predict(lo.out,
                                                         newdata=cc)

  lo.out <- loess(cong.long ~ minutes.long, span=.1, degree=2)
  cong.sum.overall <- cong.sum.overall + predict(lo.out,
                                                 newdata=xx)

  counter <- 1
  for (cval in cong.points){
    F.sum.cong[counter] <- F.sum.cong[counter] + mean(cong.long <= cval)
    counter <- counter + 1
  }
  
}

F.cong <- F.sum.cong / 10
f.cong.list <- F2f(F.cong, cong.points, width=20)

pdf("RequestByCongestion.pdf", height=9, width=9)
par(mfrow=c(2,1), mar=c(5, 4, 1, 2), lend=2)

plot(cc, cc, col="red", lwd=2, type="n", xlab="Congestion at Polling Place",
     ylab="Probability of Being Asked for ID", ylim=c(0,1), xlim=c(0,0.25))
abline(v=seq(from=0, to=0.25, by=.0125), col=rgb(0.9, 0.9, 0.9))
abline(h=seq(from=0, to=1, by=.1), col=rgb(0.9, 0.9, 0.9))
count <- 1
for (loc in locations){
  lines(cc, ask.cong.sum[[loc]]/10, col=rgb(0.5, 0.5, 0.5))
  count <- count + 1
}
lines(cc, ask.cong.sum.overall/10, col="black", lwd=4)


plot(f.cong.list$xvals, f.cong.list$fvals, type="l",
     xlab="Congestion at Polling Place",
     ylab="Density of Individual's Congestion Values",
     xlim=c(0, 0.25))
abline(h=0, col="lightgray")
dev.off()





