## calculates some simple crosstabs of voter characteristics and
## whether they were asked for ID


## raw (non-imputed) data 
raw <- read.csv("CombinedData.csv")
raw$VtrRace = NA
raw$VtrRace[raw$IsVtrAsian == 1] <- "Asian"   ## Asian
raw$VtrRace[raw$IsVtrBlack == 1] <- "Black"   ## Black
raw$VtrRace[raw$IsVtrWhite == 1] <- "White"   ## White
raw$VtrRace[raw$IsVtrOthRace == 1] <- "Other" ## OthRace

raw$MiniVtrRace = NA
raw$MiniVtrRace[raw$MiniIsVtrAsian == 1] <- "Asian"   ## Asian
raw$MiniVtrRace[raw$MiniIsVtrBlack == 1] <- "Black"   ## Black
raw$MiniVtrRace[raw$MiniIsVtrWhite == 1] <- "White"   ## White
raw$MiniVtrRace[raw$MiniIsVtrOthRace == 1] <- "Other" ## OthRace


raw$ShowID[raw$ShowID=="NO"] <- "No"
raw$ShowID[raw$ShowID=="NoMemory"] <- NA
raw$ShowID <- as.character(raw$ShowID)

## Table 1 in the article is from these 
cat("Information for Table 1 in the article:\n")
print(table(raw$ShowID, raw$VtrRace))
print(table(raw$ShowID, raw$IsVtrHisp))
cat("***************************************\n")


print(table(raw$MiniVtrRace, raw$VtrRace))
print(table(raw$MiniIsVtrHisp, raw$IsVtrHisp))
print(table(raw$ShowID, raw$ConfVtCnt, raw$VtrRace))
print(table(raw$ShowID, raw$ConfVtCnt, raw$IsVtrHisp))

raw$DR <- raw$DigResp
raw$DR[raw$DR==1] <- "(Stongly)Disagree"
raw$DR[raw$DR==2] <- "(Stongly)Disagree"
raw$DR[raw$DR==3] <- "Neutral"
raw$DR[raw$DR==4] <- "(Strongly)Agree"
raw$DR[raw$DR==5] <- "(Strongly)Agree"
print(table(raw$ShowID, raw$DR, raw$VtrRace))
print(table(raw$ShowID, raw$DR, raw$IsVtrHisp))



race <- NULL
mini.race <- NULL
showid <- NULL
hisp <- NULL
mini.hisp <- NULL
miniloc <- NULL
respondents <- NULL
confid <- NULL
digresp <- NULL

## imputed data
for (i in 0:9){
  filestring <- paste("BlueImp", i, ".csv", sep="")
  x <- read.csv(filestring, stringsAsFactors=FALSE)
  race <- c(race, as.character(x$VtrRace))
  mini.race <- c(race, as.character(x$MiniVtrRace))
  showid <- c(showid, as.character(x$ShowID))
  hisp <- c(hisp, x$IsVtrHisp)
  mini.hisp <- c(hisp, x$MiniIsVtrHisp)
  miniloc <- c(miniloc, as.character(x$MiniLoc))
  respondents <- c(respondents, rep(1, nrow(x)))
  confid <- c(confid, x$ConfVtCnt)
  digresp <- c(digresp, x$DigRespB)
  
  filestring <- paste("PinkImp", i, ".csv", sep="")
  x <- read.csv(filestring, stringsAsFactors=FALSE)
  race <- c(race, as.character(x$VtrRace))
  mini.race <- c(race, as.character(x$MiniVtrRace))
  showid <- c(showid, as.character(x$ShowID))  
  hisp <- c(hisp, x$IsVtrHisp)
  mini.hisp <- c(hisp, x$MiniIsVtrHisp)
  miniloc <- c(miniloc, as.character(x$MiniLoc))
  respondents <- c(respondents, rep(1, nrow(x)))
  confid <- c(confid, x$ConfVtCnt)
  digresp <- c(digresp, x$DigRespP)
}



cat("#########################################################\n")
cat("                  IMPUTED RESULTS           \n")


## Table 1
cat("Information for Table 1 in the article:\n")
print(table(showid, hisp)/10)
print(table(showid, race)/10)
cat("***************************************\n")

print(table(showid, confid, race)/10)
print(table(showid, confid, hisp)/10)


print(table(showid, digresp, race)/10)
print(table(showid, digresp, hisp)/10)



cat("\n")
tab1 <- table(miniloc, hisp)
tab2 <- table(miniloc, respondents)

frac.hisp <- tab1[,2] / tab2[,1]

tab1 <- table(miniloc, race)

frac.asian <- tab1[,1] / tab2[,1]
frac.black <- tab1[,2] / tab2[,1]
frac.white <- tab1[,4] / tab2[,1]



save(frac.hisp, file="frac.hisp.Rdata")
save(frac.asian, file="frac.asian.Rdata")
save(frac.black, file="frac.black.Rdata")
save(frac.white, file="frac.white.Rdata")


