
## delta is the size of the gap between inactive|white and inactive|minority
sensit.fun <- function(delta){


  ## three columns: "", "ppstrings" and "fracInac.by.pp"
  load("./PollingPlaceIDinactiveData.Rdata")
  FracInacByPP <- data.frame(pp.num=pp.data$pp.num,
                             ppstrings=pp.data$pp.string,
                             fracInac.by.pp=(pp.data$n.idinactive / pp.data$n.voters))
  
  ## returns fraction inactive by race given overall fraction and gap
  pinact <- function(finactive, nwhite, nblack, nhisp, nasian, delta){
    nvoters <- nwhite + nblack + nhisp + nasian
    fwhite <- nwhite/nvoters
    fblack <- nblack/nvoters
    fhisp <- nhisp/nvoters
    fasian <- nasian/nvoters

    fiw <- finactive - fblack*delta - fhisp*delta - fasian*delta
        
    if (fiw < 0){
      fiw <- 0
      implied.delta <- finactive / (fblack + fhisp + fasian)
      fib <- implied.delta
      fih <- implied.delta
      fia <- implied.delta
    }
    else{
      fib <- fiw + delta
      fih <- fiw + delta
      fia <- fiw + delta
    }

    return(list(fiw=fiw, fib=fib, fih=fih, fia=fia))    
  }
  

  neighbors <- function(x, window){
    nbrs <- rep(NA, length(x))
    for (i in 1:length(x)){
      nbrs[i] <- sum(x > x[i] - window & x <= x[i] + window)
    }
    return(nbrs)
  }
  
  library(MCMCpack)
  library(splines)
  
  options(warn=2)
  
  inds <- 0:9
  
  
  to.inc <- rep(NA, 39)
  sid.frac <- rep(NA, 39)
  
  for (i in inds){
    pinkstring <- paste("./PinkImp", i, ".csv", sep="")
    bluestring <- paste("./BlueImp", i, ".csv", sep="")
    
    pink <- read.csv(pinkstring, stringsAsFactors=FALSE)
    blue <- read.csv(bluestring, stringsAsFactors=FALSE) 
    
    ## Woodbourne Apartments not here "Marcus Garvey Gardens" replaces it
    treated.loc <- c("Gavin School",
                     "Boston Middle School Academy (Formally Endicott Elementary School)",
                     "Mozart School",
                     "Saint Anne's School",
                     "James A. Garfield School",
                     "St. Mark's School Hall",
                     "Roslindale Branch Library",
                     "Boston Trinity Academy ",
                     "Beethoven School",
                     "Curtis Hall" ,
                     "James J. Chittick School",
                     "Honan-Allston Branch Library",
                     "Marcus Garvey Gardens")
    
    ## Agassiz School not here "Shelburne Center"  replaces it
    control.loc <- c("Thomas A. Edison School",
                     "William Howard Taft School" ,
                     "Patrick O'Hearn School",
                     "Phineas Bates School",
                     "Lower Mills Library",
                     "George H. Conley School",
                     "Foley Building",
                     "McKinley School",
                     "Early Learning Center" ,
                     "Woodrow Wilson School",
                     "Area E Police Station" ,
                     "Thomas J Kenny School",
                     "Shelburne Center" )
    
    
    
    pink.sub <- pink[,c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                        "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")]
    blue.sub <- blue[,c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                        "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")]
    colnames(pink.sub) <- c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                            "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")
    colnames(blue.sub) <- c("ShowID", "MiniLoc", "IsVtrFem", "IsVtrHisp",
                            "VtrRace", "Educ", "VtrAge", "VtrHmLang", "MiniTime")
    
    
    
    pink.sub$treated <- pink.sub$MiniLoc %in% treated.loc
    pink.sub$control <- pink.sub$MiniLoc %in% control.loc
    pink.sub$notmatched <- !(pink.sub$MiniLoc %in% c(control.loc, treated.loc))
    pink.sub$pink <- 1
    pink.sub$SIDnum <- 1
    pink.sub$SIDnum[pink.sub$ShowID == "No"] <- 0
    pink.sub$IsVtrFem <- pink.sub$IsVtrFem == "Yes"
    pink.sub$IsVtrHisp <- pink.sub$IsVtrHisp == "Yes"
    pink.sub$IsVtrAfrAm <- pink.sub$VtrRace == "Black"
    pink.sub$IsVtrAsian <- pink.sub$VtrRace == "Asian"
    pink.sub$NonEngSpkr <- pink.sub$VtrHmLang != "Engl"
    pink.sub$CollegePlus <- pink.sub$Educ == "4YrColl" | pink.sub$Educ == "PostGrad"
    pink.sub$Young <- pink.sub$VtrAge == "<=25"
    pink.sub$Old <- pink.sub$VtrAge == ">55" 
    
    blue.sub$treated <- blue.sub$MiniLoc %in% treated.loc
    blue.sub$control <- blue.sub$MiniLoc %in% control.loc
    blue.sub$notmatched <- !(blue.sub$MiniLoc %in% c(control.loc, treated.loc))
    blue.sub$pink <- 0
    blue.sub$SIDnum <- 1
    blue.sub$SIDnum[blue.sub$ShowID == "No"] <- 0
    blue.sub$IsVtrFem <- blue.sub$IsVtrFem == "Yes"
    blue.sub$IsVtrHisp <- blue.sub$IsVtrHisp == "Yes"
    blue.sub$IsVtrAfrAm <- blue.sub$VtrRace == "Black"
    blue.sub$IsVtrAsian <- blue.sub$VtrRace == "Asian"
    blue.sub$NonEngSpkr <- blue.sub$VtrHmLang != "Engl"
    blue.sub$CollegePlus <- blue.sub$Educ == "4YrColl" | blue.sub$Educ == "PostGrad"
    blue.sub$Young <- blue.sub$VtrAge == "<=25"
    blue.sub$Old <- blue.sub$VtrAge == ">55" 
    
    mydata <- rbind(pink.sub, blue.sub)
                                        #mydata <- mydata[!mydata$notmatched,]
    drop.indic <- mydata$IsVtrHisp == FALSE & mydata$VtrRace == "OthRace"
    mydata <- mydata[!drop.indic,]
    mydata <- mydata[!is.na(mydata$MiniTime),]
    
    
    
    
    mydata$ShowID.orig <- mydata$ShowID
    
    
    ## delete observations according to various hypotheses about
    ## excess turnout
    load("./Additional-Turnout-Data.Rdata")
    counter <- 1
    nblack.newvoters <- 0
    nhisp.newvoters <- 0
    nasian.newvoters <- 0
    nwhite.newvoters <- 0
    ## loop over polling places
    for (pp in unique(add.turnout.data$name)){
      
      nvoters.pp <- sum(mydata$MiniLoc == pp)

      finactive.pp <-  FracInacByPP[FracInacByPP$ppstrings == pp, 3]
      
      nblack.pp <- sum(mydata[mydata$MiniLoc == pp, "IsVtrAfrAm"])    
      nblack.showID.pp <- sum(mydata[mydata$MiniLoc == pp &
                                     mydata$ShowID == "Yes", "IsVtrAfrAm"])
      frac.showID.given.black.pp <- nblack.showID.pp / (nblack.pp + .001)
      
      
      nhisp.pp <- sum(mydata[mydata$MiniLoc == pp, "IsVtrHisp"])    
      nhisp.showID.pp <- sum(mydata[mydata$MiniLoc == pp &
                                    mydata$ShowID == "Yes", "IsVtrHisp"])
      frac.showID.given.hisp.pp <- nhisp.showID.pp / (nhisp.pp + .001)
      
      nasian.pp <- sum(mydata[mydata$MiniLoc == pp, "IsVtrAsian"])    
      nasian.showID.pp <- sum(mydata[mydata$MiniLoc == pp &
                                     mydata$ShowID == "Yes", "IsVtrAsian"])
      frac.showID.given.asian.pp <- nasian.showID.pp / (nasian.pp + .001)
      
      
      nwhite.pp <- sum(!mydata[mydata$MiniLoc == pp, "IsVtrHisp"] &
                       !mydata[mydata$MiniLoc == pp, "IsVtrAfrAm"])    
      nwhite.showID.pp <- sum(!mydata[mydata$MiniLoc == pp &
                                      mydata$ShowID == "Yes", "IsVtrHisp"] &
                              !mydata[mydata$MiniLoc == pp &
                                      mydata$ShowID == "Yes", "IsVtrAfrAm"])
      frac.showID.given.white.pp <- nwhite.showID.pp / (nwhite.pp + .001)
      




      
      race.inactive.frac.list <- pinact(finactive.pp, nwhite.pp, nblack.pp,
                                        nhisp.pp, nasian.pp, delta)
      p.new.given.white <- race.inactive.frac.list[[1]]
      p.new.given.black <- race.inactive.frac.list[[2]]
      p.new.given.hisp <- race.inactive.frac.list[[3]]
      p.new.given.asian <- race.inactive.frac.list[[4]]


      p.new.given.white.showID.pp <- min(p.new.given.white / frac.showID.given.white.pp, 1)
      if (frac.showID.given.white.pp <=0){
        p.new.given.white.showID.pp <- 0
      }
      p.new.given.asian.showID.pp <- min(p.new.given.asian / frac.showID.given.asian.pp, 1)
      if (frac.showID.given.asian.pp <=0){
        p.new.given.asian.showID.pp <- 0
      }
      p.new.given.hisp.showID.pp <- min(p.new.given.hisp / frac.showID.given.hisp.pp, 1)
      if (frac.showID.given.hisp.pp <=0){
        p.new.given.hisp.showID.pp <- 0
      }
      p.new.given.black.showID.pp <- min(p.new.given.black / frac.showID.given.black.pp, 1)
      if (frac.showID.given.black.pp <=0){
        p.new.given.black.showID.pp <- 0
      }

      

      
      black.new.inds <- as.logical(rbinom(nblack.pp, 1, p.new.given.black.showID.pp) * (mydata[mydata$MiniLoc == pp & mydata$IsVtrAfrAm, "ShowID"] == "Yes"))
      
      hisp.new.inds <- as.logical(rbinom(nhisp.pp, 1, p.new.given.hisp.showID.pp) * (mydata[mydata$MiniLoc == pp & mydata$IsVtrHisp, "ShowID"] == "Yes"))
      
      asian.new.inds <- as.logical(rbinom(nasian.pp, 1, p.new.given.asian.showID.pp) * (mydata[mydata$MiniLoc == pp & mydata$IsVtrAsian, "ShowID"] == "Yes"))
      
      white.new.inds <- as.logical(rbinom(nwhite.pp, 1, p.new.given.white.showID.pp) * (mydata[mydata$MiniLoc == pp & !mydata$IsVtrHisp & !mydata$IsVtrAfrAm, "ShowID"] == "Yes"))
      
      
      mydata[mydata$MiniLoc == pp & mydata$IsVtrAfrAm, "SIDnum"][black.new.inds] <- NA
      
      mydata[mydata$MiniLoc == pp & mydata$IsVtrHisp, "SIDnum"][hisp.new.inds] <- NA
      
      mydata[mydata$MiniLoc == pp & mydata$IsVtrAsian, "SIDnum"][asian.new.inds] <- NA
      
      mydata[mydata$MiniLoc == pp & !mydata$IsVtrAfrAm & !mydata$IsVtrHisp, "SIDnum"][white.new.inds] <- NA
      
      
      nblack.newvoters <- nblack.newvoters + sum(black.new.inds)
      nhisp.newvoters <- nhisp.newvoters + sum(hisp.new.inds)
      nasian.newvoters <- nasian.newvoters + sum(asian.new.inds)
      nwhite.newvoters <- nwhite.newvoters + sum(white.new.inds)
      
    }
    
    mydata <- na.omit(mydata)
    
    n.newvoters <- nwhite.newvoters + nblack.newvoters + nhisp.newvoters + nasian.newvoters
    cat("Fraction New Voters Among Black Voters =    ", nblack.newvoters/sum(mydata$IsVtrAfrAm), "\n")
    cat("Fraction New Voters Among Hispanic Voters = ", nhisp.newvoters/sum(mydata$IsVtrHisp), "\n")
    cat("Fraction New Voters Among Asian Voters = ", nasian.newvoters/sum(mydata$IsVtrAsian), "\n")
    cat("Fraction New Voters Among White Voters =    ", nwhite.newvoters/sum(!(mydata$IsVtrHisp |
                                                                               mydata$IsVtrAfrAm)) , "\n\n")
    
    cat("Fraction Black Among New Voters =    ", nblack.newvoters/n.newvoters, "\n")
    cat("Fraction Hispanic Among New Voters = ", nhisp.newvoters/n.newvoters, "\n")
    cat("Fraction Asian Among New Voters = ", nasian.newvoters/n.newvoters, "\n")
    cat("Fraction White Among New Voters =    ", nwhite.newvoters/n.newvoters, "\n\n")
    
    cat("Fraction Black Voters =    ", mean(mydata$IsVtrAfrAm), "\n")
    cat("Fraction Hispanic Voters = ", mean(mydata$IsVtrHisp), "\n")
    cat("Fraction Asian Voters = ", mean(mydata$IsVtrAsian), "\n")
    cat("Fraction White Voters =    ", mean(!(mydata$IsVtrHisp |
                                              mydata$IsVtrAfrAm)) , "\n\n")
    
    cat("Fraction New Voters Among Voters = ", n.newvoters/nrow(mydata), "\n\n")
    
    
    
    cat("Imputation ", i, "\n")
    
    mydata$congestion <- NA
    locations <- unique(mydata$MiniLoc)
    
    for (loc in locations){
      mydata$congestion[mydata$MiniLoc == loc] <- neighbors(mydata$MiniTime[mydata$MiniLoc == loc], 30) / sum(mydata$MiniLoc == loc)
    }
    
    cong.spline.mat <- bs(mydata$congestion, knots=c(0.075, 0.125), degree=3,
                          Boundary.knots=c(0, 0.5))[,]
    
    y <- mydata$SIDnum
    X <- cbind(1, mydata$treated, mydata$pink, mydata$IsVtrFem, mydata$IsVtrHisp,
               mydata$IsVtrAfrAm, mydata$IsVtrAsian, mydata$NonEngSpkr,
               mydata$CollegePlus, mydata$Young, mydata$Old,
               cong.spline.mat)
    colnames(X) <- c("Intercept", "Treated", "Pink", "Female", "Hisp", "AfrAm",
                     "Asian", "NonEngSpkr", "CollegePlus", "Young", "Old",
                     "cong1", "cong2", "cong3", "cong4", "cong5")
                                        #W <- matrix(1, nrow(X), 1)
    W <- X[,c(1), drop=FALSE]
    n <- length(unique(mydata$MiniLoc))
    
    
    
    Wtilde <- matrix(0, nrow(W), n*ncol(W))
    counter <- 1
    m <- NULL
    for (loc.name in unique(mydata$MiniLoc)){
      Wtilde[mydata$MiniLoc == loc.name, counter:(counter+ncol(W)-1)] <-
        W[mydata$MiniLoc == loc.name,]
      counter <- counter + ncol(W)
      m <- c(m, sum(mydata$MiniLoc == loc.name))
    }
    
    
    
    out.basic <- MCMChieroprobit(y ~ 0+X, fullW=W, Wtilde=Wtilde, 
                                 m=m, b0=0, B0=0.000001, tune=4.5, beta.start=0,
                                 r0=3, R0=diag(.3, ncol(W)),
                                 thin=2, burnin=2000, mcmc=10000)
    
    
    
    
    eval(parse(text=paste("out.basic.", i, " <- out.basic", sep="")))
    
  }
  
  out.basic <- rbind(out.basic.0,
                     out.basic.1,
                     out.basic.2,
                     out.basic.3,
                     out.basic.4,
                     out.basic.5,
                     out.basic.6,
                     out.basic.7,
                     out.basic.8,
                     out.basic.9)
  
  out.basic <- mcmc(out.basic, thin=2)
  
  beta.hat.overall <- apply(out.basic, 2, mean)
  se.overall <- apply(out.basic, 2, sd)
  tstat <- beta.hat.overall / se.overall
  lower.025 <- apply(out.basic, 2, quantile, prob=0.025)
  upper.975 <- apply(out.basic, 2, quantile, prob=0.975)
  

  output <- c(beta.hat.overall[5], lower.025[5], upper.975[5],
              beta.hat.overall[6], lower.025[6], upper.975[6],
              beta.hat.overall[7], lower.025[7], upper.975[7])
  
  names(output) <- c("beta.hisp", "lower.hisp", "upper.hisp",
                     "beta.black", "lower.black", "upper.black",
                     "beta.asian", "lower.asian", "upper.asian")
  
  
  return(output)
  
}




