jointProbTab <- function(x, i, j){
  name1 <- colnames(x)[i]
  name2 <- colnames(x)[j]  
  table(x[,i], x[,j], dnn=c(name1, name2))  / sum(table(x[,i], x[,j]))  
}


test2way <- function(x, i, j, epsilon=0.01, cutoff=0.01){
  name1 <- colnames(x)[i]
  name2 <- colnames(x)[j]
  x1 <- x[,name1]
  x2 <- x[,name2]
  
  tab <- table(x1,x2, dnn=c(name1, name2)) + epsilon
  print(tab)
  fit.ind <- loglin(tab, margin=list(1,2), print=FALSE, fit=TRUE, param=TRUE)
  #fit.sat <- loglin(tab, margin=list(c(1,2)))

  pvalue <- 1-pchisq(fit.ind$lrt, fit.ind$df)
  cat("p-value: ", round(pvalue, 4), "\n")
  if (pvalue < cutoff){
    cat("    ", name1, " IS NOT independent of ", name2, "\n\n")
  }
  else{
    cat("    ", name1, " IS independent of ", name2, "\n\n")
  }

  return(fit.ind)
  
}


testAll2way <- function(x, epsilon=0.01, cutoff=0.01){
  margins <- NULL
  for (i in 1:(ncol(x)-1)){
    for (j in (i+1):ncol(x)){
      fit <- test2way(x, i, j, epsilon, cutoff)
      pvalue <- 1-pchisq(fit$lrt, fit$df)
      if (pvalue < cutoff){
        margins <- c(margins, i, j, 0)
      }
    }
  }
  margins <- margins[-length(margins)]
  return(margins)
}




test3way <- function(x, i, j, k, epsilon=0.01, cutoff=0.01){
  name1 <- colnames(x)[i]
  name2 <- colnames(x)[j]
  name3 <- colnames(x)[k]
  x1 <- x[,name1]
  x2 <- x[,name2]
  x3 <- x[,name3]
  
  tab <- table(x1,x2,x3, dnn=c(name1, name2,name3)) + epsilon
  print(tab)

  fit1 <- loglin(tab, margin=list(c(1,2), c(2,3)),
                 print=FALSE, fit=TRUE, param=TRUE)

  pvalue <- 1-pchisq(fit1$lrt, fit1$df)
  #cat("p-value: ", round(pvalue, 4), "\n")
  if (pvalue < cutoff){
    cat("(", name1, ", ", name2, ", ", name3, ") interaction significant\n\n")
  }
  else{
    cat("(", name1, ", ", name2, ", ", name3, ") interaction NOT significant\n\n")
  }

  return(fit1)
  
}


testAll3way <- function(x, epsilon=0.01, cutoff=0.01){
  margins <- NULL
  for (i in 1:(ncol(x)-2)){
    for (j in (i+1):(ncol(x)-1)){
      for (k in (j+1):ncol(x)){
        fit <- test3way(x, i, j, k, epsilon, cutoff)
        pvalue <- 1-pchisq(fit$lrt, fit$df)
        if (pvalue < cutoff){
          margins <- c(margins, i, j, k, 0)
        }
      }
    }
  }
  margins <- margins[-length(margins)]
  return(margins)
}



  




lowerFrom3 <- function(m, nvar){

  m.mat <- matrix(c(m, 0), nrow=4)
  marg2 <- combn(1:nvar, 2)
  marg1 <- rbind(1:nvar)

  ## get marg2
  for (i in 1:nvar){
    x1 <- marg2[1,i]
    x2 <- marg2[2,i]

    row1 <- m.mat[,m.mat[1,]==x1]
    row2 <- m.mat[,m.mat[2,]==x1]
    row3 <- m.mat[,m.mat[3,]==x1]

    if (!(x2 %in% row1) & !(x2 %in% row2) & !(x2 %in% row3) ){
      marg2[,i] <- NA
    }    
  }

  ## get marg1
  for (i in 1:nvar){
    if ( !(i %in% m.mat) ){
      marg1[i] <- NA
    }
  }

  return(list(marg3=m.mat, marg2=marg2, marg1=marg1))
  
}












## NB: this is tied tightly to the coding rules used in DataPrep.R
## any changes the coding of variables in DataPrep.R must be reflected
## with accompanying changes to OutputRecode()
##
## assumes x contains MiniColor and MiniLoc vars
##
OutputRecode <- function(x){

  
  x <- as.data.frame(x)
  
  ## recode MiniColor
  holder <- x[,"MiniColor"]
  holder[x[,"MiniColor"] == 1] <- "B"
  holder[x[,"MiniColor"] == 2] <- "G"
  holder[x[,"MiniColor"] == 3] <- "P"
  holder[x[,"MiniColor"] == 4] <- "Y"
  x[,"MiniColor"] <- holder

  ## recode MiniLoc
  load("MiniLocUnique.Rdata") ## generated by DataPrep.R
  holder <- x[,"MiniLoc"]
  for (i in 1:length(MiniLoc.uniq)){
    holder[holder == i] <- MiniLoc.uniq[i]
  }
  x[,"MiniLoc"] <- holder

  
  cnames <- colnames(x)

  for (i in 1:ncol(x)){

    holder <- x[,i]
    cname <- cnames[i]
    
    if (cname == "FmrPres3"){
      holder <- holder - 1
      holder[x[,"MiniColor"] == "Y"] <- NA
      holder[x[,"MiniColor"] == "B"] <- NA
      holder[x[,"MiniColor"] == "P"] <- NA      
      x[,i] <- holder
    }
    else if (cname == "FmrPres2"){
      holder <- holder - 1
      holder[x[,"MiniColor"] == "G"] <- NA
      holder[x[,"MiniColor"] == "B"] <- NA
      holder[x[,"MiniColor"] == "P"] <- NA      
      x[,i] <- holder
    }
    else if (cname == "Pres"){
      holder[holder==1] <- "McCain"
      holder[holder==2] <- "NoVote"
      holder[holder==3] <- "Obama"
      holder[holder==4] <- "SomeoneElse"
      holder[x[,"MiniColor"] == "B"] <- NA
      holder[x[,"MiniColor"] == "P"] <- NA      
      x[,i] <- holder
    }    
    else if (cname == "EndTax"){
      holder[holder==1] <- "NoVote"
      holder[holder==2] <- "No"
      holder[holder==3] <- "Yes"
      holder[x[,"MiniColor"] == "B"] <- NA
      holder[x[,"MiniColor"] == "P"] <- NA      
      x[,i] <- holder
    }    
    else if (cname == "EndPotPen"){
      holder[holder==1] <- "NoVote"
      holder[holder==2] <- "No"
      holder[holder==3] <- "Yes"
      holder[x[,"MiniColor"] == "B"] <- NA
      holder[x[,"MiniColor"] == "P"] <- NA      
      x[,i] <- holder
    }    
    else if (cname == "EndDogRace"){
      holder[holder==1] <- "NoVote"
      holder[holder==2] <- "No"
      holder[holder==3] <- "Yes"
      holder[x[,"MiniColor"] == "B"] <- NA
      holder[x[,"MiniColor"] == "P"] <- NA      
      x[,i] <- holder
    }    
    else if (cname == "IsVtrUSABorn"){
      holder[holder==1] <- "No"
      holder[holder==2] <- "Yes"
      holder[x[,"MiniColor"] == "B"] <- NA
      holder[x[,"MiniColor"] == "P"] <- NA      
      x[,i] <- holder
    }    
    else if (cname == "Internet"){
      holder <- holder - 1
      holder[x[,"MiniColor"] == "B"] <- NA
      holder[x[,"MiniColor"] == "P"] <- NA      
      x[,i] <- holder
    }    
    else if (cname == "Housing"){
      holder <- holder - 1
      holder[x[,"MiniColor"] == "B"] <- NA
      holder[x[,"MiniColor"] == "P"] <- NA      
      x[,i] <- holder
    }    
    else if (cname == "PrivRespP"){
      holder[holder==1] <- "Disagree"
      holder[holder==2] <- "Neutral"
      holder[holder==3] <- "Agree"
      holder[x[,"MiniColor"]== "B"] <- NA
      holder[x[,"MiniColor"]== "G"] <- NA
      holder[x[,"MiniColor"]== "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "WellRunP"){
      holder[holder==1] <- "Badly"
      holder[holder==2] <- "OK"
      holder[holder==3] <- "Well"
      holder[x[,"MiniColor"]== "B"] <- NA
      holder[x[,"MiniColor"]== "G"] <- NA
      holder[x[,"MiniColor"]== "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "DigRespP"){
      holder[holder==1] <- "Disagree"
      holder[holder==2] <- "Neutral"
      holder[holder==3] <- "Agree"
      holder[x[,"MiniColor"]== "B"] <- NA
      holder[x[,"MiniColor"]== "G"] <- NA
      holder[x[,"MiniColor"]== "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "KnewDoingP"){
      holder[holder==1] <- "Disagree"
      holder[holder==2] <- "Neutral"
      holder[holder==3] <- "Agree"
      holder[x[,"MiniColor"]== "B"] <- NA
      holder[x[,"MiniColor"]== "G"] <- NA
      holder[x[,"MiniColor"]== "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "JobPerfP"){
      holder[holder==1] <- "Poor"
      holder[holder==2] <- "Fair"
      holder[holder==3] <- "Good"
      holder[holder==4] <- "Excellent"      
      holder[x[,"MiniColor"]== "B"] <- NA
      holder[x[,"MiniColor"]== "G"] <- NA
      holder[x[,"MiniColor"]== "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "PrivRespB"){
      holder[holder==1] <- "Disagree"
      holder[holder==2] <- "Neutral"
      holder[holder==3] <- "Agree"
      holder[x[,"MiniColor"]== "P"] <- NA
      holder[x[,"MiniColor"]== "G"] <- NA
      holder[x[,"MiniColor"]== "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "WellRunB"){
      holder[holder==1] <- "Well"
      holder[holder==2] <- "OK"
      holder[holder==3] <- "Badly"
      holder[x[,"MiniColor"]== "P"] <- NA
      holder[x[,"MiniColor"]== "G"] <- NA
      holder[x[,"MiniColor"]== "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "DigRespB"){
      holder[holder==1] <- "Disagree"
      holder[holder==2] <- "Neutral"
      holder[holder==3] <- "Agree"
      holder[x[,"MiniColor"]== "P"] <- NA
      holder[x[,"MiniColor"]== "G"] <- NA
      holder[x[,"MiniColor"]== "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "KnewDoingB"){
      holder[holder==1] <- "Disagree"
      holder[holder==2] <- "Neutral"
      holder[holder==3] <- "Agree"
      holder[x[,"MiniColor"]== "P"] <- NA
      holder[x[,"MiniColor"]== "G"] <- NA
      holder[x[,"MiniColor"]== "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "JobPerfB"){
      holder[holder==1] <- "Excellent"
      holder[holder==2] <- "Good"
      holder[holder==3] <- "Fair"
      holder[holder==4] <- "Poor"
      holder[x[,"MiniColor"]== "P"] <- NA
      holder[x[,"MiniColor"]== "G"] <- NA
      holder[x[,"MiniColor"]== "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "ProbRegis"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      holder[x[,"MiniColor"] == "G"] <- NA
      holder[x[,"MiniColor"] == "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "ShowID"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      holder[holder == 3] <- "NoMemory"
      holder[x[,"MiniColor"] == "G"] <- NA
      holder[x[,"MiniColor"] == "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "DidVote"){
      holder[x[,"MiniColor"] == "G"] <- NA
      holder[x[,"MiniColor"] == "Y"] <- NA
      x[,i] <- holder
    }    
    else if (cname == "EqBalProb"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      holder[x[,"MiniColor"] == "G"] <- NA
      holder[x[,"MiniColor"] == "Y"] <- NA      
      x[,i] <- holder      
    }    
    else if (cname == "ConfVtCnt"){
      holder[x[,"MiniColor"] == "G"] <- NA
      holder[x[,"MiniColor"] == "Y"] <- NA
      x[,i] <- holder
    }    
    else if (cname == "VtrHmLang"){
      load("VtrHmLangUnique.Rdata") ## generated by DataPrep.R
      for (j in 1:length(VtrHmLang.uniq)){
        holder[holder == j] <- VtrHmLang.uniq[j]
      }
      x[,i] <- holder
    }    
    else if (cname == "IsVtrFem"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }    
    else if (cname == "MiniIsVtrFem"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }    
    else if (cname == "IsVtrHisp"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }    
    else if (cname == "MiniIsVtrHisp"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }    
    else if (cname == "VtrRace"){
      holder[holder == 1] <- "Asian"
      holder[holder == 2] <- "Black"
      holder[holder == 3] <- "White"
      holder[holder == 4] <- "OthRace"
      x[,i] <- holder
    }    
    else if (cname == "MiniVtrRace"){
      holder[holder == 1] <- "Asian"
      holder[holder == 2] <- "Black"
      holder[holder == 3] <- "White"
      holder[holder == 4] <- "OthRace"
      x[,i] <- holder
    }    
    else if (cname == "VtrAge"){
      holder[holder == 1] <- "<=25"
      holder[holder == 2] <- "26-35"
      holder[holder == 3] <- "36-45"
      holder[holder == 4] <- "46-55"
      holder[holder == 5] <- ">55"
      x[,i] <- holder
    }    
    else if (cname == "MiniVtrAge"){
      holder[holder == 1] <- "<=25"
      holder[holder == 2] <- "26-35"
      holder[holder == 3] <- "36-45"
      holder[holder == 4] <- "46-55"
      holder[holder == 5] <- ">55"
      x[,i] <- holder
    }    
    else if (cname == "Educ"){
      holder[holder == 1] <- "<=8thGrade"
      holder[holder == 2] <- "HS-GED"
      holder[holder == 3] <- "2YrColl"
      holder[holder == 4] <- "4YrColl"
      holder[holder == 5] <- "PostGrad"
      x[,i] <- holder
    }    
    else if (cname == "PstrUniv"){
      holder[holder == 1] <- "BC"
      holder[holder == 2] <- "BU"
      holder[holder == 3] <- "Harvard"
      holder[holder == 4] <- "MCPHS"
      holder[holder == 5] <- "MIT"
      holder[holder == 6] <- "MountIda"
      holder[holder == 7] <- "NELaw"
      holder[holder == 8] <- "Suffolk"
      holder[holder == 9] <- "Tufts"
      holder[holder == 10] <- "UMASS"
      holder[holder == 11] <- "Wellesley"
      x[,i] <- holder
    }    
    else if (cname == "PstrHght"){
      holder[holder == 1] <- "<=63in"
      holder[holder == 2] <- "64-66in"
      holder[holder == 3] <- "67-69in"
      holder[holder == 4] <- "70-72in"
      holder[holder == 5] <- ">72in"
      x[,i] <- holder
    }    
    else if (cname == "PstrUSABn"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }    
    else if (cname == "IsPstrFem"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }    
    else if (cname == "IsPstrHisp"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }
    else if (cname == "IsPstrAsian"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }    
    else if (cname == "IsPstrBlack"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }    
    else if (cname == "IsPstrWhite"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }    
    else if (cname == "IsPstrOthRace"){
      holder[holder == 1] <- "No"
      holder[holder == 2] <- "Yes"
      x[,i] <- holder      
    }    
    else if (cname == "PstrRace"){
      holder[holder == 1] <- "Asian"
      holder[holder == 2] <- "Black"
      holder[holder == 3] <- "White"
      holder[holder == 4] <- "OthRace"
      x[,i] <- holder
    }    


    
  } ## end of loop over columns of x

  return(x)
  
}

